<?php
namespace App\Controller;

use App\Templates;
use App\Exception;

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;

use SimpleXMLElement;

class WebService {

    private $util;
    private $integrationService;
    
    private $basePath;
    private $ordersPath;        
    private $processedPath;     
    private $errPath;
    
    private $mailList;
    private $serverName;
    
    public function __construct (\Doctrine\DBAL\Connection $connection,
                                 \App\Util\Util $util,
                                 \App\Service\IntegrationService $integrationService) {

        $this->connection         = $connection;
        $this->util               = $util;
        $this->integrationService = $integrationService;
    }
    
    public function getConsumerBalance(Request $request, Response $response) {
        try {
            
            $headers = $request->getServerParams();
            $configsFile  = file_get_contents(__DIR__ . "/../../configs/configs.json");
            $config       = json_decode($configsFile, true);
            
            $authToken = $config['authToken'];
            
            $req = $request->getParsedBody();
            
            $json = json_encode($req);
            $array = json_decode($json, TRUE);
            
            $array = $this->formatArray($array['data']);
            
            $inputXml = $req->asXML();
            
            $fileName = date("c") . ".txt";

	        $content  = "[REQUEST] - " . date("c") . "\n" . $inputXml . "\n\n";
	        
	        $this->integrationService->saveRequestLog($fileName, $content, "getConsumerBalance");
            
            if(!empty($headers['HTTP_AUTHORIZATION'])){
                
                $token = substr($headers['HTTP_AUTHORIZATION'], 7);
                
                if($token === $authToken){
                    $clientId = !empty($config['clientId']) ? $config['clientId'] : null;
                    $clientSecret = !empty($config['clientSecret']) ? $config['clientSecret'] : null;
                    
                    $requestHeader = [
                        "Content-Type: application/json",
                        "client-id: $clientId",
                        "client-secret: $clientSecret"
                    ];
                    
                    $requestBody = [
                        "cpf" => "",
                        "telefone" => $array['consumerPhone'],
                        "id_loja" => $array['storeId'],
                        "id_marca" => $array['brandId']
                    ];
                    
                    $requestResponse = $this->sendRequest("/api/Fidelidade/ConsultaPontoCliente", 'POST', $requestHeader, $requestBody);
                    $error = false;
                    
                    if(empty($requestResponse)){
                        $responseXml = $this->integrationService->buildXmlString([], 'Não foi possivel comunicar com a api GCOM.', true);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro na consulta de saldo: Não foi possivel comunicar com a api GCOM\n");
                    } 
                    
                    else if(!empty($requestResponse['status']) && $requestResponse['status'] == "ok"){
                        $formattedResponse = array(
                            [
                                "branchId" => $requestResponse['data']['id_empresa'],
                                "consumerId" => $requestResponse['data']['id_cliente'],
                                "consumerName" => $requestResponse['data']['nome_cliente'],
                                "fidelityCampaign" => $requestResponse['data']['nome_programa_fidelidade'],
                                "fidelityType" => $requestResponse['data']['tipo_fidelidade'],
                                "fidelityId" => $requestResponse['data']['id_programa_fidelidade'],
                                "fidelityVersion" => $requestResponse['data']['id_programa_fidelidade_versao'],
                                "cpf" => $requestResponse['data']['cpf'],
                                "phone" => $requestResponse['data']['ddd'] . $requestResponse['data']['telefone'],
                                "balance" => $requestResponse['data']['valor_cashback_saldo'],
                                "expiredbalance" => $requestResponse['data']['valor_cashback_expirado'],
                                "redemptionPercentage" => $requestResponse['data']['pc_limite_resgate_vl_vnd'],
                                "creditPercentage" => $requestResponse['data']['percentual_valor'],
                                "status" => $requestResponse['data']['cadastrado'],
                                "brandId" => $requestResponse['data']['id_marca']
                            ]
                        );
                        $responseXml = $this->integrationService->buildXmlString($formattedResponse, '', false);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Consulta de saldo realizada com sucesso.\n");
                    }
                    
                    else if(!empty($requestResponse['message'])){
                        $responseXml = $this->integrationService->buildXmlString($requestResponse, $requestResponse['message'], true);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro na consulta de saldo: " . $requestResponse['message'] . "\n");
                    }
                    
                    else{
                        $responseXml = $this->integrationService->buildXmlString($requestResponse, "Ocorreu um erro inesperado.", true);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro na consulta de saldo: Erro inesperado.\n");
                    }
                    
                    $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
                    $this->integrationService->saveRequestLog($fileName, $content, "getConsumerBalance");
                    
                    return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
                } else{
                    $responseXml = $this->integrationService->buildXmlString([], 'Token de autenticacao invalido', true);
                    $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Token de autenticacao invalido.\n");
                    
                    $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
                    $this->integrationService->saveRequestLog($fileName, $content, "getConsumerBalance");
                    
                    return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
                }
            } else{
                $responseXml = $this->integrationService->buildXmlString([], 'Token de autenticacao nao enviado', true);
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Token de autenticacao nao enviado.\n");
                
                $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
                $this->integrationService->saveRequestLog($fileName, $content, "getConsumerBalance");
                
                return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
            }
            
        } catch (\Exception $e) {
            $responseXml = $this->integrationService->buildXmlString([], 'Ocorreu um erro inesperado', true);
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Ocorreu um erro inesperado: " . $e->getMessage() . "\n");
            
            $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
            $this->integrationService->saveRequestLog($fileName, $content, "getConsumerBalance");
            
            return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
        }
    }
    
    public function balanceReserve(Request $request, Response $response) {
        try {
            $headers = $request->getServerParams();
            $configsFile  = file_get_contents(__DIR__ . "/../../configs/configs.json");
            $config       = json_decode($configsFile, true);
            
            $authToken = $config['authToken'];
            
            $req = $request->getParsedBody();
            
            $json = json_encode($req);
            $array = json_decode($json, TRUE);
            
            $array = $this->formatArray($array['data']);
            
            $inputXml = $req->asXML();
            
            $fileName = date("c") . ".txt";

	        $content  = "[REQUEST] - " . date("c") . "\n" . $inputXml . "\n\n";
	        
	        $this->integrationService->saveRequestLog($fileName, $content, "balanceReserve");
            
            if(!empty($headers['HTTP_AUTHORIZATION'])){
                
                $token = substr($headers['HTTP_AUTHORIZATION'], 7);
                
                if($token === $authToken){
                    $clientId = !empty($config['clientId']) ? $config['clientId'] : null;
                    $clientSecret = !empty($config['clientSecret']) ? $config['clientSecret'] : null;
                    
                    $requestHeader = [
                        "Content-Type: application/json",
                        "client-id: $clientId",
                        "client-secret: $clientSecret"
                    ];
                    
                    $requestBody = [
                        "valor_reserva" => $array['value'],
                        "id_empresa" => $array['branchId'],
                        "id_programa_fidelidade" => $array['fidelityId'],
                        "id_programa_fidelidade_versao" => $array['fildetyVersion'],
                        "id_marca" => $array['brandId'],
                        "reserva_multipla" => false,
                        "id_loja" => $array['storeId']
                    ];
                    
                    $requestResponse = $this->sendRequest("/api/Fidelidade/Cliente/".$array['consumerId']."/GerarReserva", 'POST', $requestHeader, $requestBody);
                    $error = false;
                    
                    if(empty($requestResponse)){
                        $responseXml = $this->integrationService->buildXmlString([], 'Não foi possivel comunicar com a api GCOM.', true);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro na reserva de saldo: Não foi possivel comunicar com a api GCOM\n");
                    } 
                    
                    else if(!empty($requestResponse['status']) && $requestResponse['status'] == "ok"){
                        $formattedResponse = array(
                            [
                                "reserveId" => $requestResponse['data']['id_reserva']
                            ]
                        );
                        $responseXml = $this->integrationService->buildXmlString($formattedResponse, '', false);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Reserva de saldo realizada com sucesso.\n");
                    }
                    
                    else if(!empty($requestResponse['message'])){
                        $responseXml = $this->integrationService->buildXmlString($requestResponse, $requestResponse['message'], true);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro na reserva de saldo: " . $requestResponse['message'] . "\n");
                    }
                    
                    else{
                        $responseXml = $this->integrationService->buildXmlString($requestResponse, "Ocorreu um erro inesperado.", true);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro na reserva de saldo: Erro inesperado.\n");
                    }
                    
                    $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
                    $this->integrationService->saveRequestLog($fileName, $content, "balanceReserve");
                    
                    return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
                } else{
                    $responseXml = $this->integrationService->buildXmlString([], 'Token de autenticacao invalido', true);
                    $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Token de autenticacao invalido.\n");
                    
                    $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
                    $this->integrationService->saveRequestLog($fileName, $content, "balanceReserve");
                    
                    return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
                }
            } else{
                $responseXml = $this->integrationService->buildXmlString([], 'Token de autenticacao nao enviado', true);
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Token de autenticacao nao enviado.\n");
                
                $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
                $this->integrationService->saveRequestLog($fileName, $content, "balanceReserve");
                
                return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
            }
            
        } catch (\Exception $e) {
            $responseXml = $this->integrationService->buildXmlString([], 'Ocorreu um erro inesperado', true);
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Ocorreu um erro inesperado: " . $e->getMessage() . "\n");
            
            $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
            $this->integrationService->saveRequestLog($fileName, $content, "balanceReserve");
            
            return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
        }
    }
    
    public function deleteReserve(Request $request, Response $response) {
        try {
            $headers = $request->getServerParams();
            $configsFile  = file_get_contents(__DIR__ . "/../../configs/configs.json");
            $config       = json_decode($configsFile, true);
            
            $authToken = $config['authToken'];
            
            $req = $request->getParsedBody();
            
            $json = json_encode($req);
            $array = json_decode($json, TRUE);
            
            $array = $this->formatArray($array['data']);
            
            $inputXml = $req->asXML();
            
            $fileName = date("c") . ".txt";

	        $content  = "[REQUEST] - " . date("c") . "\n" . $inputXml . "\n\n";
	        
	        $this->integrationService->saveRequestLog($fileName, $content, "deleteReserve");
            
            if(!empty($headers['HTTP_AUTHORIZATION'])){
                
                $token = substr($headers['HTTP_AUTHORIZATION'], 7);
                
                if($token === $authToken){
                    $clientId = !empty($config['clientId']) ? $config['clientId'] : null;
                    $clientSecret = !empty($config['clientSecret']) ? $config['clientSecret'] : null;
                    
                    $requestHeader = [
                        "Content-Type: application/json",
                        "client-id: $clientId",
                        "client-secret: $clientSecret"
                    ];
                    
                    $requestBody = [
                        "id_reserva" => $array['reserveId']
                    ];
                    
                    $requestResponse = $this->sendRequest("/api/Fidelidade/Cliente/".$array['consumerId']."/ExcluirReserva", 'POST', $requestHeader, $requestBody);
                    $error = false;
                    
                    if(empty($requestResponse)){
                        $responseXml = $this->integrationService->buildXmlString([], 'Não foi possivel comunicar com a api GCOM.', true);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro na exclusao de reserva de saldo: Não foi possivel comunicar com a api GCOM\n");
                    } 
                    
                    else if(!empty($requestResponse['status']) && $requestResponse['status'] == "ok"){
                        $formattedResponse = array([]);
                        $responseXml = $this->integrationService->buildXmlString($formattedResponse, '', false);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Exclusao de reserva de saldo realizada com sucesso.\n");
                    }
                    
                    else if(!empty($requestResponse['message'])){
                        $responseXml = $this->integrationService->buildXmlString($requestResponse, $requestResponse['message'], true);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro na exclusao de reserva de saldo: " . $requestResponse['message'] . "\n");
                    }
                    
                    else{
                        $responseXml = $this->integrationService->buildXmlString($requestResponse, "Ocorreu um erro inesperado.", true);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro na exclusao reserva de saldo: Erro inesperado.\n");
                    }
                    
                    $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
                    $this->integrationService->saveRequestLog($fileName, $content, "deleteReserve");
                    
                    return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
                } else{
                    $responseXml = $this->integrationService->buildXmlString([], 'Token de autenticacao invalido', true);
                    $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Token de autenticacao invalido.\n");
                    
                    $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
                    $this->integrationService->saveRequestLog($fileName, $content, "deleteReserve");
                    
                    return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
                }
            } else{
                $responseXml = $this->integrationService->buildXmlString([], 'Token de autenticacao nao enviado', true);
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Token de autenticacao nao enviado.\n");
                
                $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
                $this->integrationService->saveRequestLog($fileName, $content, "deleteReserve");
                
                return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
            }
            
        } catch (\Exception $e) {
            $responseXml = $this->integrationService->buildXmlString([], 'Ocorreu um erro inesperado', true);
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Ocorreu um erro inesperado: " . $e->getMessage() . "\n");
            
            $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
            $this->integrationService->saveRequestLog($fileName, $content, "deleteReserve");
            
            return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
        }
    }
    
    public function generateBalance(Request $request, Response $response) {
        try {
            $headers = $request->getServerParams();
            $configsFile  = file_get_contents(__DIR__ . "/../../configs/configs.json");
            $config       = json_decode($configsFile, true);
            
            $authToken = $config['authToken'];
            
            $req = $request->getParsedBody();
            
            $json = json_encode($req);
            $array = json_decode($json, TRUE);
            
            $array = $this->formatArray($array['data']);
            
            $inputXml = $req->asXML();
            
            $fileName = date("c") . ".txt";

	        $content  = "[REQUEST] - " . date("c") . "\n" . $inputXml . "\n\n";
	        
	        $this->integrationService->saveRequestLog($fileName, $content, "generateBalance");
            
            if(!empty($headers['HTTP_AUTHORIZATION'])){
                
                $token = substr($headers['HTTP_AUTHORIZATION'], 7);
                
                if($token === $authToken){
                    $clientId = !empty($config['clientId']) ? $config['clientId'] : null;
                    $clientSecret = !empty($config['clientSecret']) ? $config['clientSecret'] : null;
                    
                    $requestHeader = [
                        "Content-Type: application/json",
                        "client-id: $clientId",
                        "client-secret: $clientSecret"
                    ];
                    
                    $requestBody = [
                        "id_marca" => $array['brandId'],
                        "cnpj_loja" => $array['cnpj'],
                        "origem_pedido" => $this->formatOrigin($array['orderOrigin'], $array['isTakeOut'], $array['subProjectId']),
                        "numero_pedido" => $array['saleCode'],
                        "numero_pedido_web" => $array['saleExtCode'],
                        "data_pedido" => $array['saleDate'],
                        "valor_pedido" => $array['totalValue'],
                        "valor_elegivel_geracao_cashback" => $array['eligibleValue'],
                        "taxa_entrega" => $array['deliveryFee'],
                        "taxa_servico" => $array['serviceFee'],
                        "desconto" => $array['discount'],
                        "produtos" => $this->formatProducts($array['products']),
                        "cliente" => [
                            "nome" => $array['client']['consumerName'],
                            "cpf" => $array['client']['cpf'],
                            "telefone" => $array['client']['phone'],
                            "email" => ''
                        ],
                        "cashback" => [
                            "id_reserva" => $array['cashback']['reserveId'],
                            "usou_cashback" => $array['cashback']['usedCashback'],
                            "valor_cashback_utz" => $array['cashback']['usedCashbackValue']
                        ]
                    ];
                    
                    $requestResponse = $this->sendRequest("/api/Fidelidade/GerarCashback", 'POST', $requestHeader, $requestBody);
                    $error = false;
                    
                    if(empty($requestResponse)){
                        $responseXml = $this->integrationService->buildXmlString([], 'Não foi possivel comunicar com a api GCOM.', true);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro ao gerar saldo: Não foi possivel comunicar com a api GCOM\n");
                    } 
                    
                    else if(!empty($requestResponse['status']) && $requestResponse['status'] == "ok"){
                        $formattedResponse = array([]);
                        $responseXml = $this->integrationService->buildXmlString($formattedResponse, '', false);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Geracao de saldo realizada com sucesso.\n");
                    }
                    
                    else if(!empty($requestResponse['message'])){
                        $responseXml = $this->integrationService->buildXmlString($requestResponse, $requestResponse['message'], true);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro ao gerar saldo: " . $requestResponse['message'] . "\n");
                    }
                    
                    else{
                        $responseXml = $this->integrationService->buildXmlString($requestResponse, "Ocorreu um erro inesperado.", true);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro ao gerar saldo: Erro inesperado.\n");
                    }
                    
                    $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
                    $this->integrationService->saveRequestLog($fileName, $content, "generateBalance");
                    
                    return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
                } else{
                    $responseXml = $this->integrationService->buildXmlString([], 'Token de autenticacao invalido', true);
                    $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Token de autenticacao invalido.\n");
                    
                    $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
                    $this->integrationService->saveRequestLog($fileName, $content, "generateBalance");
                    
                    return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
                }
            } else{
                $responseXml = $this->integrationService->buildXmlString([], 'Token de autenticacao nao enviado', true);
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Token de autenticacao nao enviado.\n");
                
                $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
                $this->integrationService->saveRequestLog($fileName, $content, "generateBalance");
                
                return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
            }
            
        } catch (\Exception $e) {
            $responseXml = $this->integrationService->buildXmlString([], 'Ocorreu um erro inesperado', true);
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Ocorreu um erro inesperado: " . $e->getMessage() . "\n");
            
            $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
            $this->integrationService->saveRequestLog($fileName, $content, "generateBalance");
            
            return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
        }
    }
    
    public function cancelBalance(Request $request, Response $response) {
        try {
            $headers = $request->getServerParams();
            $configsFile  = file_get_contents(__DIR__ . "/../../configs/configs.json");
            $config       = json_decode($configsFile, true);
            
            $authToken = $config['authToken'];
            
            $req = $request->getParsedBody();
            
            $json = json_encode($req);
            $array = json_decode($json, TRUE);
            
            $array = $this->formatArray($array['data']);
            
            $inputXml = $req->asXML();
            
            $fileName = date("c") . ".txt";

	        $content  = "[REQUEST] - " . date("c") . "\n" . $inputXml . "\n\n";
	        
	        $this->integrationService->saveRequestLog($fileName, $content, "cancelBalance");
            
            if(!empty($headers['HTTP_AUTHORIZATION'])){
                
                $token = substr($headers['HTTP_AUTHORIZATION'], 7);
                
                if($token === $authToken){
                    $clientId = !empty($config['clientId']) ? $config['clientId'] : null;
                    $clientSecret = !empty($config['clientSecret']) ? $config['clientSecret'] : null;
                    
                    $requestHeader = [
                        "Content-Type: application/json",
                        "client-id: $clientId",
                        "client-secret: $clientSecret"
                    ];
                    
                    $requestBody = [
                        "id_marca" => $array['brandId'],
                        "cnpj_loja" => $array['cnpj'],
                        "numero_pedido" => $array['saleCode'],
                        "numero_pedido_web" => $array['saleExtCode']
                    ];
                    
                    $requestResponse = $this->sendRequest("/api/Fidelidade/EstornarCashback", 'POST', $requestHeader, $requestBody);
                    $error = false;
                    
                    if(empty($requestResponse)){
                        $responseXml = $this->integrationService->buildXmlString([], 'Não foi possivel comunicar com a api GCOM.', true);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro no cancelamento de saldo: Não foi possivel comunicar com a api GCOM\n");
                    } 
                    
                    else if(!empty($requestResponse['status']) && $requestResponse['status'] == "ok"){
                        $formattedResponse = array([]);
                        $responseXml = $this->integrationService->buildXmlString($formattedResponse, '', false);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Exclusao de saldo realizada com sucesso.\n");
                    }
                    
                    else if(!empty($requestResponse['message'])){
                        $responseXml = $this->integrationService->buildXmlString($requestResponse, $requestResponse['message'], true);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro no cancelamento de saldo: " . $requestResponse['message'] . "\n");
                    }
                    
                    else{
                        $responseXml = $this->integrationService->buildXmlString($requestResponse, "Ocorreu um erro inesperado.", true);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro no cancelamento de saldo: Erro inesperado.\n");
                    }
                    
                    $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
                    $this->integrationService->saveRequestLog($fileName, $content, "saveRequestLog");
                    
                    return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
                } else{
                    $responseXml = $this->integrationService->buildXmlString([], 'Token de autenticacao invalido', true);
                    $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Token de autenticacao invalido.\n");
                    
                    $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
                    $this->integrationService->saveRequestLog($fileName, $content, "saveRequestLog");
                    
                    return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
                }
            } else{
                $responseXml = $this->integrationService->buildXmlString([], 'Token de autenticacao nao enviado', true);
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Token de autenticacao nao enviado.\n");
                
                $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
                $this->integrationService->saveRequestLog($fileName, $content, "saveRequestLog");
                
                return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
            }
            
        } catch (\Exception $e) {
            $responseXml = $this->integrationService->buildXmlString([], 'Ocorreu um erro inesperado', true);
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Ocorreu um erro inesperado: " . $e->getMessage() . "\n");
            
            $content = "[RESPONSE] - " . date("c") . "\n" . $responseXml;
            
            $this->integrationService->saveRequestLog($fileName, $content, "saveRequestLog");
            
            return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
        }
    }
    
    private function formatProducts($productsArray){
        $formattedProducts = [];
        
        foreach($productsArray as $product){
            $auxArray = [
                "nome"=> $product['productName'],
                "sku"=> $product['productCode'],
                "valor"=> $product['value'],
                "quantidade"=> $product['quantity'],
                "desconto"=> $product['discount']
            ];
            
            array_push($formattedProducts, $auxArray);
        }
        
        return $formattedProducts;
    }
    
    private function formatOrigin($orderOrigin, $isTakeOut, $subProjectId) {
        if($orderOrigin == 'DLV_INT'){
            return 'FOS';
        }
        
        else if($orderOrigin == 'DLV_FOS'){
            return 'FOS';
        }
        
        else if($orderOrigin == 'TGO_TAA'){
            return 'DINE IN TAA TEKNISA';
        }
        
        else if($orderOrigin == 'BAL_TAA'){
            return 'DINE IN TAA TEKNISA';
        }
        
        else if($orderOrigin == 'MES_FOS' || $orderOrigin == 'BAL_FOS' || $orderOrigin == 'CMD_FOS'){
            return 'BALCAO';
        }
        
        else if($orderOrigin == 'DLV_IFO' && $isTakeOut == 'N'){
            return 'DELIVERY IFOOD';
        }
        
        else if($orderOrigin == 'DLV_IFO' && $isTakeOut == 'S'){
            return 'RETIRADA IFOOD';
        }
        
        else if($orderOrigin == 'DLV_NMO' && $isTakeOut == 'N'){
            return 'DELIVERY NEEMO';
        }
        
        else if($orderOrigin == 'DLV_NMO' && $isTakeOut == 'S'){
            return 'RETIRADA NEEMO';
        }
        
        else if($orderOrigin == 'DLV_NMO' && $isTakeOut == 'C'){
            return 'DINE IN NEEMO';
        }
        
        else if($orderOrigin == 'DLV_RAP' && $isTakeOut == 'N'){
            return 'DELIVERY RAPPI';
        }
        
        else if($orderOrigin == 'DLV_RAP' && $isTakeOut == 'S'){
            return 'RETIRADA RAPPI';
        }
        
        else if($orderOrigin == 'DLV_ANT' && $isTakeOut == 'N'){
            return 'DELIVERY ANOTA AI';
        }
        
        else if($orderOrigin == 'DLV_ANT' && $isTakeOut == 'S'){
            return 'RETIRADA ANOTA AI';
        }
        
        else if($orderOrigin == 'DLV_ANT' && $isTakeOut == 'C'){
            return 'DINE IN ANOTA AI';
        }
        
        else if($orderOrigin == 'TGO_PED' && $isTakeOut == 'N'){
            return 'DELIVERY PEDE PRONTO';
        }
        
        else if($orderOrigin == 'TGO_PED' && $isTakeOut == 'S'){
            return 'RETIRADA PEDE PRONTO';
        }
        
        else if($orderOrigin == 'BAL_PED' && $isTakeOut == 'C'){
            return 'DINE IN PEDE PRONTO';
        }
        
        else if($orderOrigin == 'DLV_DDI' && $isTakeOut == 'N'){
            return 'DELIVERY DELIVERY DIRETO';
        }
        
        else if($orderOrigin == 'DLV_DDI' && $isTakeOut == 'S'){
            return 'RETIRADA DELIVERY DIRETO';
        }
        
        else if($orderOrigin == 'DLV_DDI' && $isTakeOut == 'C'){
            return 'DINE IN DELIVERY DIRETO';
        }
        
        else{
            throw new \Exception('Nao foi possivel identificar a origem do pedido pelos dados enviados');
        }
    }
    
    private function sendRequest($route, $method, $requestHeader, $requestBody, $query = null){
        $configsFile  = file_get_contents(__DIR__ . "/../../configs/configs.json");
        $config       = json_decode($configsFile, true);
        
        $url = !empty($config['baseURL']) ? $config['baseURL'] : null;
        
        $requestUrl = $url.$route;
        
        if(!empty($query)){
            $requestUrl = $requestUrl.'?'.$query;
        }
        
        $curl = curl_init();
        
        if(empty($requestBody)){
            curl_setopt_array($curl, [
                CURLOPT_URL             => $requestUrl,
                CURLOPT_RETURNTRANSFER  => true,
                CURLOPT_ENCODING        => '',
                CURLOPT_CONNECTTIMEOUT  => 5,
                CURLOPT_TIMEOUT         => 90,
                CURLOPT_FOLLOWLOCATION  => true,
                CURLOPT_SSL_VERIFYPEER  => false,
                CURLOPT_USERAGENT       => "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:17.0) Gecko/20100101 Firefox/17.0",
                CURLOPT_HTTP_VERSION    => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST   => $method,
                CURLOPT_HTTPHEADER      => $requestHeader
            ]);
        } else{
            curl_setopt_array($curl, [
                CURLOPT_URL             => $requestUrl,
                CURLOPT_RETURNTRANSFER  => true,
                CURLOPT_ENCODING        => '',
                CURLOPT_CONNECTTIMEOUT  => 5,
                CURLOPT_TIMEOUT         => 90,
                CURLOPT_FOLLOWLOCATION  => true,
                CURLOPT_SSL_VERIFYPEER  => false,
                CURLOPT_USERAGENT       => "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:17.0) Gecko/20100101 Firefox/17.0",
                CURLOPT_HTTP_VERSION    => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST   => $method,
                CURLOPT_POSTFIELDS      => json_encode($requestBody),
                CURLOPT_HTTPHEADER      => $requestHeader
            ]);
        }
        
        $curlResponse = curl_exec($curl);
        $httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);

        if (!empty(curl_error($curl)) || ($httpCode != 200) || (stripos($curlResponse, "<html>")) || (stripos($curlResponse, "Fatal error"))) {
            $fileName = "EXP-".date("H-i-s") . ".txt";

            $this->util->writeLog('network', $fileName, $this->util->formatNetworkError([
                'eventTime' => date("d/m/Y H:i:s"),
                'header'    => $requestHeader,
                'url'       => $url.$route,
                'method'    => $method,
                'params'    => json_encode($requestBody),
                'httpCode'  => $httpCode,
                'error'     => curl_error($curl),
                'response'  => $curlResponse
            ]));
        }
        
        curl_close($curl);
        
        return json_decode($curlResponse, true);
    }

    private function writeEventsLog($events) {
        foreach($events as $event) {

            $reference = $event['correlationId'];
            $eventCode = $event['code'];

            $fileName = round(microtime(true)*1000) . "-$eventCode.json";
            $this->util->writeLog("events", $fileName, $event);
        }
    }
    
    public function welcome(Request $request, Response $response) {
        $sql        = "SELECT CDFILIAL FROM FILIAL";
        $result     = $this->connection->fetchAll($sql);

        $versionInfo = $this->getVersionInfo();

        return $response->withJson([
            'version'     => $versionInfo['version'],
            'revision'    => $versionInfo['revision'],
            'date'        => $versionInfo['date'],
            'branchs'     => count($result)
        ]);
    }

    private function getVersionInfo() {
        $file = file_get_contents(__DIR__ . "/../../version.json");
        return json_decode($file, true);
    }

    private function getClientName() {
        global $CLIENT_NAME;
        return $CLIENT_NAME;
    }
    
    private function formatArray($array) {
        $formattedArray = [];
        
        foreach($array as $arrayElement){
            $indexName = array_search($arrayElement, $array);
            
            if(is_array($arrayElement)){
                if($indexName === 'data' || $indexName === 'prod'){
                    if((sizeof($arrayElement) > 1) && (!empty($arrayElement[0]))){
                        foreach($arrayElement as $arrElemChild){
                            if(is_array($arrElemChild)){
                                array_push($formattedArray, $this->formatArray($arrElemChild));
                            } else{
                                array_push($formattedArray, $arrElemChild);
                            }
                        }
                    } else{
                        array_push($formattedArray, $this->formatArray($arrayElement));
                    }
                } else{
                    $formattedArray[$indexName] = $this->formatArray($arrayElement);
                }
            } else{
                if($indexName === 'data' || $indexName === 'prod'){
                    array_push($formattedArray, $arrayElement);
                } else{
                    $formattedArray[$indexName] = $arrayElement;
                }
            }
            
            unset($array[$indexName]);
        }
        
        return $formattedArray;
    }
}
